/*
 * (c) Copyright 2004        Martin Hejl <martin@hejl.de>
 *                           G&H Softwareentwicklung GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>

#define PROC_ERROR_LED "/proc/driver/soekris_error_led"
#define PROC_DIRECTION "/proc/driver/soekris_io_settings"
#define PROC_GPIO      "/proc/driver/soekris_gpio"

#define BUF_SIZE 4096

void write_to_fd(int fd, char* buf, int len) {
	write(fd, buf,len);
	fsync(fd);
}

void read_from_fd(int fd, char* buf, int len) {
	int readlen=0;
	lseek(fd,0,SEEK_SET);
	//memset(buf,0,BUF_SIZE);
	readlen = read(fd, buf, len);
	buf[readlen]=0;
	//fprintf(stderr, "Read %d bytes \n%s\n", readlen, buf);
}

int main (int argc, char *argv[])
{
	int use4801 =0;
	char buffer[BUF_SIZE+1];
	int fProcErrorLED;
	int fProcDirection;
	int fProcGPIO;
	char* toTest;

	buffer[BUF_SIZE] = 0;

	/* Tests for both 45xx and 4801 */

	/* Open the "file" in the proc system, which provides us with the */
	/* required information about the network devices */
	fProcErrorLED = open(PROC_ERROR_LED, O_RDWR);
	if (fProcErrorLED == -1) {
		fprintf(stderr, "Unable to open %s.", PROC_ERROR_LED);
		return 1;

	}

	fProcDirection = open(PROC_DIRECTION, O_RDWR);
	if (fProcDirection == -1) {
		fprintf(stderr, "Unable to open %s.", PROC_DIRECTION);
		return 1;
	}

	fProcGPIO = open(PROC_GPIO, O_RDWR);
	if (fProcGPIO == -1) {
		fprintf(stderr, "Unable to open %s.", PROC_DIRECTION);
		return 1;
	}

	/* find out it we're dealing with a 4801 */
	read_from_fd(fProcGPIO,buffer, BUF_SIZE);
	if (strlen(buffer)>= 10) {
		use4801=1;
	}

	// write 1 to error led
	write_to_fd(fProcErrorLED , "1",1);

	// read 1 from error led
	read_from_fd(fProcErrorLED,buffer, BUF_SIZE);
	fprintf(stderr, "Writing 1 to /proc/driver/soekris_error_led              %s\n", buffer[0] == '1'?"ok":"not ok");

	// write 0 to error led
	write_to_fd(fProcErrorLED , "0",1);

	// read 0 from error led
	read_from_fd(fProcErrorLED,buffer, BUF_SIZE);
	fprintf(stderr, "Writing 0 to /proc/driver/soekris_error_led              %s\n",buffer[0] == '0'?"ok":"not ok");


	close(fProcErrorLED);

	if (use4801==0) {
		/* Tests for 45xx only */
		// set direction to 00000000
		toTest = "00000000\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 00000000 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 11111111
		toTest = "11111111\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 11111111 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 10101010
		toTest = "10101010\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 10101010 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 01010101
		toTest = "01010101\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 01010101 to /proc/driver/soekris_gpio %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// Set everything to output
		toTest = "11111111\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));


		// write 00000000 to GPIO
		toTest = "00000000\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 00000000 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 11111111 to GPIO
		toTest = "11111111\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 11111111 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 10101010 to GPIO
		toTest = "10101010\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 10101010 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 01010101 to GPIO
		toTest = "01010101\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 01010101 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");


		// Set everything to input
		// the way the GPIOs are connected on the 4501, we expect 11110000
		// no matter what we write to the port
		toTest = "00000000\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));

		// write 00000000 to GPIO
		toTest = "00000000\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 00000000 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"11110000\n")==0?"ok":"not ok");

		// write 11111111 to GPIO
		toTest = "11111111\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 11111111 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"11110000\n")!=0?"ok":"not ok");


		// write 10101010 to GPIO
		toTest = "10101010\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 10101010 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"11110000\n")!=0?"ok":"not ok");

		// write 01010101 to GPIO
		toTest = "01010101\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 01010101 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"11110000\n")!=0?"ok":"not ok");

	} else {
		/* Tests for 48xx only */

		// set direction to 000000000000
		toTest = "000000000000\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 000000000000 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 111111111111
		toTest = "111111111111\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 111111111111 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 101010101010
		toTest = "101010101010\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 101010101010 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// set direction to 010101010101
		toTest = "010101010101\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));
		read_from_fd(fProcDirection,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 010101010101 to /proc/driver/soekris_io_settings %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// Set everything to output
		toTest = "111111111111\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));


		// write 000000000000 to GPIO
		toTest = "000000000000\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 000000000000 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 111111111111 to GPIO
		toTest = "111111111111\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 111111111111 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 101010101010 to GPIO
		toTest = "101010101010\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 101010101010 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");

		// write 010101010101 to GPIO
		toTest = "010101010101\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 010101010101 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,toTest)==0?"ok":"not ok");


		// Set everything to input
		// since all GPIOS are configures as low-active, we expect 111111111111
		// no matter what we write to the port
		toTest = "000000000000\n";
		write_to_fd(fProcDirection, toTest,strlen(toTest));

		// write 000000000000 to GPIO
		toTest = "000000000000\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 000000000000 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"111111111111\n")==0?"ok":"not ok");


		// write 111111111111 to GPIO
		toTest = "111111111111\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 111111111111 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"111111111111\n")==0?"ok":"not ok");


		// write 101010101010 to GPIO
		toTest = "101010101010\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 101010101010 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"111111111111\n")==0?"ok":"not ok");


		// write 010101010101 to GPIO
		toTest = "010101010101\n";
		write_to_fd(fProcGPIO, toTest,strlen(toTest));
		read_from_fd(fProcGPIO,buffer, BUF_SIZE);
		fprintf(stderr, "Writing 010101010101 to /proc/driver/soekris_gpio        %s\n", strcmp(buffer,"111111111111\n")==0?"ok":"not ok");

	}

	return 0;
}
